/**
 * @(#)GameInfo.java
 * 
 *  Copyright Anthony Yakovlev <yakovlev@rain.ifmo.ru> and Michail Lukin <michail@users.msn.com>
 */

package ru.ifmo.knim.main;

/**
 * Represents game info
 * @author Anthony Yakovlev
 */
public class GameInfo {
	/**
	 * Current state of game
	 */
	private int lines [];
	
	/**
	 * Limit values for all lines
	 */
	private int rlines [];
	
	/**
	 * Amount of lines of stones in current game
	 */
	private int nLines;
	
	/**
	 * The value of difficulty
	 */
	private int difficulty;
	
	/**
	 * The amount of phora stones left 
	 */
	private int badMovesLeft;
	
	/**
	 * "Easy" level of difficulty
	 */
	public static final int DIFFICULTY_EASY = 0;
	
	/**
	 * "Medium" level of difficulty
	 */
	public static final int DIFFICULTY_MEDIUM = 1;
	
	/**
	 * "Hard" level of difficulty
	 */
	public static final int DIFFICULTY_HARD = 2;
	
	/**
	 * No difficulty at all
	 */
	public static final int DIFFICULTY_NONE = -239;
	
	MoveResult  latestMove;
	
	/**
	 * Creates a gameinfo object
	 * @param nLines - amount of lines in a game
	 */
	public GameInfo (int nLines) {
		lines = new int [nLines];
		this.nLines = nLines;
		restartGame();
		latestMove = new MoveResult();
		difficulty = DIFFICULTY_EASY;
	}
	
	/**
	 * non-default contructor
	 * @param stones
	 */
	public GameInfo (int [] stones) {
		setStones(stones);
		latestMove = new MoveResult();
		difficulty = DIFFICULTY_EASY;
	}
	
	/**
	 * Gets array of fish lines
	 * @return
	 */
	public final int [] getLines () {
		return (int [])lines.clone();		
	}
	
	/**
	 * Set array of stones as rules for new game
	 * @param stones
	 */
	public void setStones(int [] stones) {
		nLines = stones.length;
		lines = new int [nLines];
		rlines = new int [nLines];
		System.arraycopy(stones, 0, lines, 0, nLines);
		System.arraycopy(stones, 0, rlines, 0, nLines);
	}
	
	/**
	 * Restarts game.
	 */
	public synchronized void restartGame () {
		badMovesLeft = 2 - difficulty;
		System.arraycopy(rlines, 0, lines, 0, nLines);
	}
	
	/**
	 * Very cool function -- analog of compareAndSet primitive
	 * @param difficulty
	 * @return
	 */
	public synchronized final int setGetDifficulty (int difficulty) {
		if (difficulty < DIFFICULTY_EASY || difficulty > DIFFICULTY_HARD ) {		
			return this.difficulty;
		} else {
			this.difficulty = difficulty;
			return difficulty;
		}
	}
	
	/**
	 * Returns the amount of lines with stones
	 * @return value from description
	 */
	public synchronized int getLineAmount () {
		return nLines;		
	}
	
	/**
	 * Returns amount of fish in a line
	 * @param lineIdx - the line idx
	 * @return amount of stones
	 */
	public synchronized int getFishInLine(int lineIdx) {
		return lines [lineIdx];				
	}
	
	/**
	 * Takes the given amount of stones from the given line
	 * @param lineIdx - the line idx
	 * @param amount - amount of stones to take
	 */
	public synchronized void takeStones (int lineIdx, int amount) {
		lines [lineIdx] -= amount;		
	}
	
	/**
	 * Returns the amount of phora moves left
	 * @return the amount of phora moves
	 */
	public int getBadMoveLeft() {
		return badMovesLeft;		
	}
	
	/**
	 * Sets the amount of phora moves
	 * @param badMovesLeft the amount of phora moves
	 */
	public void setBadMovesLeft(int badMovesLeft) {
		this.badMovesLeft = badMovesLeft;
	}
	
	/**
	 * Detects whether the game is over or not
	 * @return true if the game is over
	 */
	public boolean isGameover() {
		for (int i = 0; i < getLineAmount(); i++) {
			if (getFishInLine(i) > 0) {
				return false;
			}
		}
		return true;
	}
}
