/**
 * @(#)GamePlay.java
 * 
 *  Copyright Anthony Yakovlev <yakovlev@rain.ifmo.ru> and Michail Lukin <michail@users.msn.com>
 */

package ru.ifmo.knim.main;

import ru.ifmo.knim.providers.GameEventProvider;

import com.evelopers.unimod.core.stateworks.Event;
import com.evelopers.unimod.runtime.ModelEngine;
import com.evelopers.unimod.runtime.context.Parameter;
import com.evelopers.unimod.runtime.context.StateMachineContextImpl;

/**
 * Something that aggregates gameInfo and modelEngine.
 * @author Anthony Yakovlev
 */
public class GamePlay {
	/**
	 * Singleton object
	 */
	static GamePlay gamePlay = new GamePlay();
	
	/**
	 * Game information
	 */
	GameInfo	gameInfo;
	
	/**
	 * Signal model engine
	 */
	ModelEngine	modelEngine;
	
	/**
	 * Default constructor 
	 */
	public GamePlay () {
		int [] moves = {3, 4, 5};
		gameInfo = new GameInfo(moves);
	}
	
	/**
	 * Initializes singleton object
	 * @param modelEngine
	 */
	public void init(ModelEngine modelEngine) {
		this.modelEngine = modelEngine;		
	}
	
	/**
	 * Returns singleton object
	 * @return gamePlay object
	 */
	public static GamePlay getGamePlay () {
		return gamePlay;
	}
	
	/**
	 * Gets game information
	 * @return the game information
	 */
	public GameInfo getGameInfo() {
		return gameInfo;
	}
	
	/**
	 * Fires event to apropriate handler
	 * @param eventName - the event name
	 * @param parameter - the parameters
	 */
	private void fireEvent(String eventName, Parameter parameter) {
		if (modelEngine != null) {
			if (parameter == null) {
				modelEngine.getEventManager().handle(new Event(eventName), StateMachineContextImpl.create());
			} else {
				modelEngine.getEventManager().handle(new Event(eventName, new Parameter[]{parameter}), StateMachineContextImpl.create());
			}
		}
	}
	
	/**
	 * Restarts game
	 */
	public void restart () {
		gameInfo.restartGame();		
	}
	
	/**
	 * Starts AI automata to process current situation
	 */
	public void aiStartup () {
		int xor = computeCurrentSumm();
		fireEvent(GameEventProvider.E21,  new Parameter("XORSumm", new Integer(xor)));		
	}

	/**
	 * Computes current magic summ
	 * @return the magic summ
	 */
	private int computeCurrentSumm() {
		int xorValue = 0; 
		for(int i = 0; i < gameInfo.getLineAmount(); i++){
			xorValue ^= gameInfo.getFishInLine(i);
		}
		return xorValue;
	}

	/**
	 * Computes and makes move using the safe strategy
	 */
	public void sendSafeMove () {
		// generate result
		MoveResult result = new MoveResult();
		for(int i = 0; i < gameInfo.getLineAmount(); i++){
			if(gameInfo.getFishInLine(i) != 0){
				result.take(i, 1);
				break;
			}
		}
		
		// send out
		fireEvent(GameEventProvider.E22, new Parameter("AIMove", result));
	}

	/**
	 * Resends specifired move data
	 */
	public void resendMove(MoveResult move) {
		fireEvent(GameEventProvider.E20, new Parameter("AIMove", move));
	}
	
	/**
	 * Computes and makes move using the unsafe strategy
	 */
	public void sendUnsafeMove () {
		// generate result
		if(gameInfo.getBadMoveLeft() > 0){
			gameInfo.setBadMovesLeft(gameInfo.getBadMoveLeft() - 1);
			sendSafeMove();
		} else{
			MoveResult result = new MoveResult();
			int xorValue = computeCurrentSumm();
			for(int i = 5; i >= 0; i--){
				if(( xorValue & (1 << i) ) != 0){
					for(int j = 0; j < gameInfo.getLineAmount(); j++){
						int nStones = gameInfo.getFishInLine(j); 
						if(( nStones & (1 << i) ) != 0){//We found a line.
							int amount = nStones ^ xorValue;
							amount = nStones - amount;
							result.take(j, amount);
							break;
						}
					}
					break;
				}
			}
			
			// send out
			fireEvent(GameEventProvider.E22, new Parameter("AIMove", result));
		}
	
	}
}
